<?php

declare (strict_types=1);
/**
 * PSR-7 StreamInterface implementation
 */
namespace OM4\CopyCraft\Vendor\Art4\Requests\Psr;

use OM4\CopyCraft\Vendor\Psr\Http\Message\StreamInterface;
use RuntimeException;
use OM4\CopyCraft\Vendor\WpOrg\Requests\Exception\InvalidArgument;
/**
 * PSR-7 StreamInterface implementation
 *
 * Describes a data stream.
 *
 * Typically, an instance will wrap a PHP stream; this interface provides
 * a wrapper around the most common operations, including serialization of
 * the entire stream to a string.
 */
final class StringBasedStream implements StreamInterface
{
    /**
     * Create StringBasedStream from string
     *
     * @param string $content
     * @return static
     */
    public static function createFromString($content)
    {
        if (!\is_string($content)) {
            throw InvalidArgument::create(1, '$content', 'string', \gettype($content));
        }
        return new self($content);
    }
    /**
     * @var string
     */
    private $content;
    /**
     * Constructor
     *
     * @param content $content
     */
    private function __construct($content)
    {
        if (!\is_string($content)) {
            throw InvalidArgument::create(1, '$content', 'content', \gettype($content));
        }
        $this->content = $content;
    }
    /**
     * Reads all data from the stream into a string, from the beginning to end.
     *
     * This method MUST attempt to seek to the beginning of the stream before
     * reading data and read the stream until the end is reached.
     *
     * Warning: This could attempt to load a large amount of data into memory.
     *
     * This method MUST NOT raise an exception in order to conform with PHP's
     * string casting operations.
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString()
    {
        return $this->content;
    }
    /**
     * Closes the stream and any underlying resources.
     *
     * @return void
     */
    public function close()
    {
        return;
    }
    /**
     * Separates any underlying resources from the stream.
     *
     * After the stream has been detached, the stream is in an unusable state.
     *
     * @return resource|null Underlying PHP stream, if any
     */
    public function detach()
    {
        return null;
    }
    /**
     * Get the size of the stream if known.
     *
     * @return int|null Returns the size in bytes if known, or null if unknown.
     */
    public function getSize()
    {
        return \strlen($this->content);
    }
    /**
     * Returns the current position of the file read/write pointer
     *
     * @return int Position of the file pointer
     * @throws \RuntimeException on error.
     */
    public function tell()
    {
        throw new RuntimeException(__METHOD__ . '() is not implemented.');
    }
    /**
     * Returns true if the stream is at the end of the stream.
     *
     * @return bool
     */
    public function eof()
    {
        return \true;
    }
    /**
     * Returns whether or not the stream is seekable.
     *
     * @return bool
     */
    public function isSeekable()
    {
        return \false;
    }
    /**
     * Seek to a position in the stream.
     *
     * @link http://www.php.net/manual/en/function.fseek.php
     * @param int $offset Stream offset
     * @param int $whence Specifies how the cursor position will be calculated
     *     based on the seek offset. Valid values are identical to the built-in
     *     PHP $whence values for `fseek()`.  SEEK_SET: Set position equal to
     *     offset bytes SEEK_CUR: Set position to current location plus offset
     *     SEEK_END: Set position to end-of-stream plus offset.
     * @throws \RuntimeException on failure.
     */
    public function seek($offset, $whence = \SEEK_SET)
    {
        throw new RuntimeException(__METHOD__ . '() is not implemented.');
    }
    /**
     * Seek to the beginning of the stream.
     *
     * If the stream is not seekable, this method will raise an exception;
     * otherwise, it will perform a seek(0).
     *
     * @see seek()
     * @link http://www.php.net/manual/en/function.fseek.php
     * @throws \RuntimeException on failure.
     */
    public function rewind()
    {
        throw new RuntimeException(__METHOD__ . '() is not implemented.');
    }
    /**
     * Returns whether or not the stream is writable.
     *
     * @return bool
     */
    public function isWritable()
    {
        return \false;
    }
    /**
     * Write data to the stream.
     *
     * @param string $string The string that is to be written.
     * @return int Returns the number of bytes written to the stream.
     * @throws \RuntimeException on failure.
     */
    public function write($string)
    {
        throw new RuntimeException(__METHOD__ . '() is not implemented.');
    }
    /**
     * Returns whether or not the stream is readable.
     *
     * @return bool
     */
    public function isReadable()
    {
        return \false;
    }
    /**
     * Read data from the stream.
     *
     * @param int $length Read up to $length bytes from the object and return
     *     them. Fewer than $length bytes may be returned if underlying stream
     *     call returns fewer bytes.
     * @return string Returns the data read from the stream, or an empty string
     *     if no bytes are available.
     * @throws \RuntimeException if an error occurs.
     */
    public function read($length)
    {
        throw new RuntimeException(__METHOD__ . '() is not implemented.');
    }
    /**
     * Returns the remaining contents in a string
     *
     * @return string
     * @throws \RuntimeException if unable to read or an error occurs while
     *     reading.
     */
    public function getContents()
    {
        throw new RuntimeException(__METHOD__ . '() is not implemented.');
    }
    /**
     * Get stream metadata as an associative array or retrieve a specific key.
     *
     * The keys returned are identical to the keys returned from PHP's
     * stream_get_meta_data() function.
     *
     * @link http://php.net/manual/en/function.stream-get-meta-data.php
     * @param string $key Specific metadata to retrieve.
     * @return array|mixed|null Returns an associative array if no key is
     *     provided. Returns a specific key value if a key is provided and the
     *     value is found, or null if the key is not found.
     */
    public function getMetadata($key = null)
    {
        if (\func_num_args() > 0 && !\is_string($key)) {
            throw InvalidArgument::create(1, '$key', 'string', \gettype($key));
        }
        if (\is_string($key)) {
            return null;
        }
        return [];
    }
}
